#!/usr/bin/env python3
# Enhanced scan report generator - Uses PID data to create enhanced reports

import os
import sys
import re
import json
import glob
from datetime import datetime
import traceback

# Path where scan reports are stored
SCAN_DIRS = [
    "/media/usb/ServiceScan",
    "/hdd/ServiceScan",
    "/media/FTA/ServiceScan",
    "/tmp"
]

def parse_scan_report(filepath):
    """Extract transponder information and services from ServiceScan XML report"""
    transponders = []
    frequencies = []
    services_by_tp = {}  # To store services organized by transponder number
    satellite = ""
    
    # Simple regex-based parsing since the files aren't strictly XML
    with open(filepath, 'r') as f:
        content = f.read()
        
        # Get satellite name
        sat_match = re.search(r'<\s*Satellite\s*=\s*([^>]+)\s*>', content)
        if sat_match:
            satellite = sat_match.group(1).strip()
        
        # Extract transponder information - handle both formats we've seen
        tp_matches = re.findall(r"<\s*'(DVB-S2?[^']+)'\s*>\s*Tp#\s*(\d+)", content)
        
        # If no matches found, try alternative pattern seen in real reports
        if not tp_matches:
            tp_matches = re.findall(r"<\s*'(DVB-S2\s+\w+\s+\d+[HV]\s*/\s*\d+\s*/\s*[\d/]+)'\s*>\s*Tp#\s*(\d+)", content)
        
        # Process each transponder
        for tp_info, tp_num in tp_matches:
            # Extract frequency and polarization
            freq_match = re.search(r'(\d+)([HV])', tp_info)
            if freq_match:
                freq = freq_match.group(1)
                pol = freq_match.group(2)
                
                # Add to list of frequencies for TSDuck analysis
                frequencies.append((freq, pol, tp_num))
                
                # Create transponder object
                tp_obj = {
                    'number': tp_num,
                    'frequency': freq,
                    'polarization': pol,
                    'full_info': tp_info,
                    'services': []  # Will hold services for this transponder
                }
                
                # Find the section of content for this transponder
                tp_start = content.find(f"Tp# {tp_num}")
                if tp_start != -1:
                    # Find the end of this transponder's content (next transponder or end of file)
                    next_tp = content.find("Tp#", tp_start + 6)
                    if next_tp == -1:
                        tp_content = content[tp_start:]
                    else:
                        tp_content = content[tp_start:next_tp]
                    
                    # Extract services using improved pattern that correctly matches actual format
                    # Pattern: < [xx] Channel_Name  (Radio #1)  1:0:2:300:1:1:A450FA9:0:0:0: SNR = 11.82  />
                    service_pattern = r"<\s*\[(\d+)\]\s+([^(]+?)\s+(?:\(([^)]+)\))?\s+([0-9a-fA-F:]+):\s*SNR\s*=\s*([0-9.]+)"
                    service_matches = re.findall(service_pattern, tp_content)
                    
                    print(f"Found {len(service_matches)} services in transponder {tp_num}")
                    
                    # Process each service in this transponder
                    for idx, name, service_type, service_ref, snr in service_matches:
                        # Clean up values
                        name = name.strip()
                        service_type = service_type.strip()
                        service_ref = service_ref.strip()
                        
                        # Determine service type
                        if service_type.startswith("Radio"):
                            type_str = "Radio"
                        elif "TV" in service_type or ":1:" in service_ref:
                            type_str = "Television"
                        elif ":2:" in service_ref:
                            type_str = "Radio"
                        else:
                            type_str = "Data"
                        
                        # Create service object
                        service = {
                            'id': idx,
                            'name': name,
                            'type': type_str,
                            'reference': service_ref,
                            'snr': float(snr.strip()),
                            'provider': satellite  # Use satellite name as provider
                        }
                        
                        # Add service to the transponder
                        tp_obj['services'].append(service)
                    
                # Add the transponder with its services to the list
                transponders.append(tp_obj)
                
                # Store services by transponder number for easy lookup
                services_by_tp[tp_num] = tp_obj['services']
    
    return {
        'satellite': satellite,
        'transponders': transponders,
        'frequencies': frequencies,
        'services_by_tp': services_by_tp
    }

def load_pid_data(scan_file):
    """Load PID data from the JSON file if available"""
    pid_file = None
    
    # Check for a PID data file for this scan
    if scan_file.endswith('.xml'):
        pid_file = scan_file.replace('.xml', '.pids.json')
    else:
        pid_file = scan_file + '.pids.json'
    
    if os.path.exists(pid_file):
        try:
            with open(pid_file, 'r') as f:
                pid_data = json.load(f)
            print(f"Loaded PID data from {pid_file}")
            return pid_data
        except Exception as e:
            print(f"Error loading PID data: {e}")
    
    print(f"No PID data file found: {pid_file}")
    return None

def enhance_transponder_with_pid_data(tp_data, pid_data):
    """Enhance a transponder with real PID data if available"""
    # Find the matching transponder in the PID data
    tp_key = f"{tp_data['frequency']}-{tp_data['polarization']}"
    
    if not pid_data or 'transponders' not in pid_data or tp_key not in pid_data['transponders']:
        print(f"No PID data available for transponder {tp_key}")
        return None
        
    # Get the PID data for this transponder
    tp_pid_data = pid_data['transponders'][tp_key]
    
    # Create a map of services in the PID data by name and ID
    if 'services' in tp_pid_data:
        pid_services_by_name = {svc['service_name'].strip(): svc for svc in tp_pid_data['services']}
        pid_services_by_id = {svc['service_id']: svc for svc in tp_pid_data['services']}
        
        # Map services in tp_data to services in pid_data
        for service in tp_data['services']:
            service_name = service.get('name', '').strip()
            service_id = service.get('id')
            
            # Find matching service in PID data
            pid_service = None
            if service_name and service_name in pid_services_by_name:
                pid_service = pid_services_by_name[service_name]
            elif service_id and int(service_id) in pid_services_by_id:
                pid_service = pid_services_by_id[int(service_id)]
            
            if pid_service:
                # Add PID information to the service
                service['pmt_pid'] = pid_service.get('pmt_pid')
                service['elementary_streams'] = pid_service.get('elementary_streams', [])
                service['pid_data_available'] = True
    
    # Add signal metrics if available
    if 'signal_metrics' in tp_pid_data:
        tp_data['signal_metrics'] = tp_pid_data['signal_metrics']
    
    # Add symbol rate if available
    if 'symbol_rate' in tp_pid_data:
        tp_data['symbol_rate'] = tp_pid_data['symbol_rate']
    
    return tp_data

def create_enhanced_report(scan_file, scan_data, pid_data=None):
    """Create an enhanced scan report using PID data if available"""
    # Determine output file paths
    base_path = os.path.dirname(scan_file)
    base_name = os.path.basename(scan_file)
    
    if base_name.endswith('.xml'):
        html_output = os.path.join(base_path, base_name.replace('.xml', '.enhanced.html'))
        txt_output = os.path.join(base_path, base_name.replace('.xml', '.enhanced.txt'))
    else:
        html_output = os.path.join(base_path, base_name + '.enhanced.html')
        txt_output = os.path.join(base_path, base_name + '.enhanced.txt')
    
    # Copy files to /tmp for maximum visibility
    tmp_html = os.path.join("/tmp", os.path.basename(html_output))
    tmp_txt = os.path.join("/tmp", os.path.basename(txt_output))
    
    # Create HTML report
    html = f"""<!DOCTYPE html>
<html>
<head>
    <title>Enhanced Scan Report</title>
    <style>
        body {{ font-family: Arial, sans-serif; margin: 20px; }}
        h1 {{ color: #244a7c; }}
        .transponder {{ margin-bottom: 20px; border: 1px solid #ddd; padding: 10px; }}
        .transponder-header {{ background-color: #f5f5f5; padding: 5px; margin-bottom: 10px; }}
        .service {{ margin-left: 20px; border-bottom: 1px solid #eee; padding: 5px; }}
        .pid-data {{ background-color: #f9f9f9; padding: 10px; margin-top: 10px; }}
        .error {{ color: red; }}
        .signal-meter {{ width: 100px; height: 15px; background-color: #eee; margin: 5px 0; }}
        .signal-value {{ height: 100%; background-color: #4CAF50; }}
        table {{ border-collapse: collapse; width: 100%; }}
        th, td {{ border: 1px solid #ddd; padding: 8px; text-align: left; }}
        th {{ background-color: #f2f2f2; }}
        tr:nth-child(even) {{ background-color: #f9f9f9; }}
    </style>
</head>
<body>
    <h1>Enhanced Scan Report</h1>
    <p><strong>Original Scan:</strong> {os.path.basename(scan_file)}</p>
    <p><strong>Satellite:</strong> {scan_data['satellite']}</p>
    <p><strong>Transponders Found:</strong> {len(scan_data['transponders'])}</p>
    <p><strong>Analysis Date:</strong> {datetime.now().strftime("%Y-%m-%d %H:%M:%S")}</p>
    
    <h2>Transponder Details</h2>
"""
    
    # Text report header
    txt_report = f"""
=======================================================================
Enhanced Scan Report
=======================================================================
Original Scan: {os.path.basename(scan_file)}
Satellite: {scan_data['satellite']}
Transponders Found: {len(scan_data['transponders'])}
Analysis Date: {datetime.now().strftime("%Y-%m-%d %H:%M:%S")}

=======================================================================
Transponder Details
=======================================================================
"""
    
    # Process each transponder
    for tp in scan_data['transponders']:
        # Try to enhance with PID data if available
        if pid_data:
            enhance_transponder_with_pid_data(tp, pid_data)
        
        # Add to HTML report
        html += f"""<div class="transponder">
    <div class="transponder-header">
        <h3>Transponder #{tp['number']} - {tp['frequency']}{tp['polarization']}</h3>
        <p>{tp['full_info']}</p>
    </div>
"""
        
        # Add to text report
        txt_report += f"\n-----------------------------------------------------------------------\n"
        txt_report += f"Transponder #{tp['number']} - {tp['frequency']}{tp['polarization']}\n"
        txt_report += f"{tp['full_info']}\n"
        
        # Add PID data section if we have real PIDs
        has_real_pids = any('pid_data_available' in svc for svc in tp['services'])
        
        if has_real_pids:
            html += '<div class="pid-data">'
            html += '<h4>PID Analysis</h4>'
            
            txt_report += "\nPID Analysis:\n"
            
            # Add signal metrics if available
            if 'signal_metrics' in tp:
                signal = tp['signal_metrics']
                html += '<h5>Signal Information:</h5>'
                html += '<div style="display: flex; gap: 20px;">'
                
                txt_report += "\nSignal Information:\n"
                
                if 'signal_strength' in signal:
                    strength = int(signal['signal_strength'])
                    db_value = round(strength / 5, 1)
                    html += f'<div><p>Signal Strength: {strength}% ({db_value} dB)</p>'
                    html += f'<div class="signal-meter"><div class="signal-value" style="width: {strength}%;"></div></div></div>'
                    
                    txt_report += f"Signal Strength: {strength}% ({db_value} dB)\n"
                
                if 'signal_quality' in signal:
                    quality = int(signal['signal_quality'])
                    quality_db = round(quality / 5, 1)
                    html += f'<div><p>Signal Quality: {quality}% ({quality_db} dB)</p>'
                    html += f'<div class="signal-meter"><div class="signal-value" style="width: {quality}%;"></div></div></div>'
                    
                    txt_report += f"Signal Quality: {quality}% ({quality_db} dB)\n"
                
                if 'snr' in signal:
                    html += f'<div><p>SNR: {signal["snr"]} dB</p></div>'
                    txt_report += f"SNR: {signal['snr']} dB\n"
                
                html += '</div>'
            
            # Add services with PIDs
            html += '<h5>Services and PIDs:</h5>'
            txt_report += "\nServices and PIDs:\n"
            
            for service in tp['services']:
                html += f'<div class="service">'
                html += f'<h6>{service["name"]} (ID: {service["id"]} - {service["type"]})</h6>'
                
                txt_report += f"\n{service['name']} (ID: {service['id']} - {service['type']})\n"
                
                if 'pid_data_available' in service:
                    if 'pmt_pid' in service:
                        html += f'<p>PMT PID: 0x{service["pmt_pid"]:04X} ({service["pmt_pid"]})</p>'
                        txt_report += f"  PMT PID: 0x{service['pmt_pid']:04X} ({service['pmt_pid']})\n"
                    
                    if 'elementary_streams' in service and service['elementary_streams']:
                        html += '<table>'
                        html += '<tr><th>Stream Type</th><th>PID</th><th>Description</th></tr>'
                        
                        txt_report += "  Elementary Streams:\n"
                        txt_report += "  Type      | PID       | Description\n"
                        txt_report += "  ----------+-----------+----------------------------------\n"
                        
                        for stream in service['elementary_streams']:
                            stream_type = stream.get('type', 'Unknown')
                            pid = stream.get('pid', 0)
                            description = f"{stream_type} ({stream.get('stream_type', '')})"
                            
                            html += f'<tr><td>{stream_type}</td><td>0x{pid:04X} ({pid})</td><td>{description}</td></tr>'
                            txt_report += f"  {stream_type:<10}| 0x{pid:04X} ({pid:<5})| {description}\n"
                        
                        html += '</table>'
                else:
                    html += '<p>No PID data available for this service</p>'
                    txt_report += "  No PID data available for this service\n"
                
                html += '</div>'
            
            html += '</div>'
        else:
            html += '<p>No PID data available for this transponder</p>'
            txt_report += "\nNo PID data available for this transponder\n"
        
        html += '</div>'
    
    html += """</body>
</html>"""
    
    # Write the reports
    with open(html_output, 'w') as f:
        f.write(html)
    
    with open(txt_output, 'w') as f:
        f.write(txt_report)
    
    # Also write to /tmp for visibility
    with open(tmp_html, 'w') as f:
        f.write(html)
    
    with open(tmp_txt, 'w') as f:
        f.write(txt_report)
    
    # Set proper permissions
    for file_path in [html_output, txt_output, tmp_html, tmp_txt]:
        try:
            os.chmod(file_path, 0o644)
        except:
            pass
    
    # Force a sync to ensure files are written
    os.system('sync')
    
    print(f"Enhanced reports created: {html_output} and {txt_output}")
    return (html_output, txt_output)

def find_latest_scan_file():
    """Find the latest scan file in any of the standard directories"""
    latest_file = None
    latest_time = 0
    
    for dir_path in SCAN_DIRS:
        if not os.path.exists(dir_path):
            continue
            
        # Look for XML scan files
        for file_path in glob.glob(os.path.join(dir_path, "*.xml")):
            # Check if this is a scan report file (not just any XML)
            try:
                with open(file_path, 'r') as f:
                    first_line = f.readline().strip()
                    if "Scan Report" not in first_line:
                        continue
            except:
                continue
            
            # Check if this is newer than our current latest
            file_time = os.path.getmtime(file_path)
            if file_time > latest_time:
                latest_time = file_time
                latest_file = file_path
    
    return latest_file

def main():
    """Main function to process a scan file"""
    # Set up logging
    import logging
    log_file = "/tmp/enhance_scan.log"
    logging.basicConfig(
        filename=log_file,
        level=logging.INFO,
        format='%(asctime)s - %(levelname)s - %(message)s'
    )
    
    # Also log to console
    console = logging.StreamHandler()
    console.setLevel(logging.INFO)
    logging.getLogger('').addHandler(console)
    
    logging.info("Starting enhance_scan.py")
    
    # Check for command-line argument (specific scan file)
    if len(sys.argv) > 1:
        scan_file = sys.argv[1]
        if not os.path.exists(scan_file):
            logging.error(f"Scan file does not exist: {scan_file}")
            print(f"Error: Scan file does not exist: {scan_file}")
            sys.exit(1)
    else:
        # Find the latest scan file
        scan_file = find_latest_scan_file()
        if not scan_file:
            logging.info("No scan files found")
            print("No scan files found")
            sys.exit(0)
    
    logging.info(f"Processing scan file: {scan_file}")
    
    try:
        # Parse the scan report
        scan_data = parse_scan_report(scan_file)
        logging.info(f"Found {len(scan_data['transponders'])} transponders")
        
        # Try to load PID data if available
        pid_data = load_pid_data(scan_file)
        
        # Create enhanced reports
        html_output, txt_output = create_enhanced_report(scan_file, scan_data, pid_data)
        
        logging.info(f"Enhanced reports created: {html_output} and {txt_output}")
        
        # Mark as processed
        with open(f"{scan_file}.enhanced", "w") as f:
            f.write(f"Enhanced at {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}\n")
        
    except Exception as e:
        logging.error(f"Error processing scan file: {e}")
        logging.error(traceback.format_exc())
        print(f"Error processing scan file: {e}")
        sys.exit(1)

if __name__ == "__main__":
    main()