#!/bin/sh
#
# Simple, direct script for the PID Scanner plugin
# Directly calls pid_extractor_poc without interactive prompts
#

# Default output directory
OUTPUT_DIR="/tmp"

# Start log file
LOG_FILE="${OUTPUT_DIR}/pid_scan_direct.log"
echo "Direct PID scan started at $(date)" > "$LOG_FILE"

# Process Options
STOP_ENIGMA2=1  # Default: stop Enigma2

# Check for no-stop option (for use with properly written frontend management plugins)
if [ "$1" = "--no-stop" ]; then
    STOP_ENIGMA2=0
    echo "No-stop mode detected - will not restart Enigma2" >> "$LOG_FILE"
    shift  # Remove the option from the argument list
fi

# Check arguments
if [ $# -lt 7 ]; then
    echo "Usage: $0 [--no-stop] <freq_mhz> <polarity> <lo_freq_mhz> <system> <symbolrate> <modulation> <fec>" | tee -a "$LOG_FILE"
    echo "Example: $0 4180 0 5750 1 30000000 0 0" | tee -a "$LOG_FILE"
    echo "Example with no-stop: $0 --no-stop 4180 0 5750 1 30000000 0 0" | tee -a "$LOG_FILE"
    exit 1
fi

# Get parameters
TRUE_FREQ_MHZ=$1
POLARITY=$2
LO_FREQ_MHZ=$3
SYSTEM=$4
SYMBOL_RATE=$5
MODULATION=$6
FEC=$7

# Log received parameters
echo "Parameters received:" >> "$LOG_FILE"
echo "Frequency: $TRUE_FREQ_MHZ MHz" >> "$LOG_FILE"
echo "Polarity: $POLARITY (0=H, 1=V)" >> "$LOG_FILE"
echo "LNB LO: $LO_FREQ_MHZ MHz" >> "$LOG_FILE"
echo "System: $SYSTEM (0=DVB-S, 1=DVB-S2)" >> "$LOG_FILE"
echo "Symbol Rate: $SYMBOL_RATE symbols/sec" >> "$LOG_FILE"
echo "Modulation: $MODULATION (0=QPSK, 1=8PSK)" >> "$LOG_FILE"
echo "FEC: $FEC" >> "$LOG_FILE"

# Calculate tuning frequency based on band type
if [ "$TRUE_FREQ_MHZ" -lt 5000 ]; then
    # C-band uses low-side injection
    TUNING_FREQ_KHZ=$(( (LO_FREQ_MHZ - TRUE_FREQ_MHZ) * 1000 ))
    BAND_TYPE="C-band"
else
    # Ku-band uses high-side injection
    TUNING_FREQ_KHZ=$(( (TRUE_FREQ_MHZ - LO_FREQ_MHZ) * 1000 ))
    BAND_TYPE="Ku-band"
fi

# Log calculated tuning frequency
echo "Band type: $BAND_TYPE" >> "$LOG_FILE"
echo "Calculated tuning frequency: $TUNING_FREQ_KHZ kHz" >> "$LOG_FILE"

# Generate timestamp for filenames
TIMESTAMP=$(date +"%Y-%m-%d_%H-%M-%S")
RESULTS_FILE="${OUTPUT_DIR}/${BAND_TYPE}_results_${TRUE_FREQ_MHZ}MHz_${TIMESTAMP}.txt"
PARAMS_FILE="${OUTPUT_DIR}/${BAND_TYPE}_params_${TRUE_FREQ_MHZ}MHz_${TIMESTAMP}.txt"

# Save parameters to file
{
    echo "==================================================="
    echo "${BAND_TYPE} Transponder Parameters"
    echo "==================================================="
    echo "True downlink frequency: $TRUE_FREQ_MHZ MHz"
    echo "Polarization: $([ "$POLARITY" = "0" ] && echo "Horizontal" || echo "Vertical")"
    echo "LNB LO frequency: $LO_FREQ_MHZ MHz"
    echo "Calculated tuning frequency: $TUNING_FREQ_KHZ kHz"
    echo "System: $([ "$SYSTEM" = "0" ] && echo "DVB-S" || echo "DVB-S2")"
    echo "Symbol rate: $SYMBOL_RATE symbols/sec"
    echo "Modulation: $([ "$MODULATION" = "0" ] && echo "QPSK" || echo "8PSK")"
    case $FEC in
        0) echo "FEC: AUTO" ;;
        1) echo "FEC: 1/2" ;;
        2) echo "FEC: 2/3" ;;
        3) echo "FEC: 3/4" ;;
        4) echo "FEC: 5/6" ;;
        5) echo "FEC: 7/8" ;;
        6) echo "FEC: 8/9" ;;
        7) echo "FEC: 3/5" ;;
        8) echo "FEC: 4/5" ;;
        9) echo "FEC: 9/10" ;;
        *) echo "FEC: Unknown" ;;
    esac
    echo "Timestamp: $(date)"
    echo "==================================================="
} > "$PARAMS_FILE"

# Find pid_extractor_poc - check in standard locations
PID_EXTRACTOR=""
if [ -x "./pid_extractor_poc" ]; then
    PID_EXTRACTOR="./pid_extractor_poc"
elif [ -x "/usr/bin/pid_extractor_poc" ]; then
    PID_EXTRACTOR="/usr/bin/pid_extractor_poc"
elif [ -x "/bin/pid_extractor_poc" ]; then
    PID_EXTRACTOR="/bin/pid_extractor_poc"
elif [ -x "/usr/local/bin/pid_extractor_poc" ]; then
    PID_EXTRACTOR="/usr/local/bin/pid_extractor_poc"
elif command -v pid_extractor_poc >/dev/null 2>&1; then
    PID_EXTRACTOR="pid_extractor_poc"
else
    echo "Error: pid_extractor_poc tool not found in standard locations." | tee -a "$LOG_FILE"
    # For debugging, let's see what's in /usr/bin
    echo "Contents of /usr/bin (grep for pid):" | tee -a "$LOG_FILE"
    ls -la /usr/bin | grep -i pid | tee -a "$LOG_FILE"
    # Also check if package is properly installed
    echo "Checking if package is installed:" | tee -a "$LOG_FILE"
    opkg list-installed | grep pid | tee -a "$LOG_FILE"
    exit 1
fi

# Conditionally stop Enigma2 to free up the frontend
if [ $STOP_ENIGMA2 -eq 1 ]; then
    echo "Stopping Enigma2 service to free frontend..." | tee -a "$LOG_FILE"
    init 4
    sleep 3
else
    echo "Not stopping Enigma2 (frontend management handled by calling application)..." | tee -a "$LOG_FILE"
    
    # Check for frontend information file
    if [ -f "${OUTPUT_DIR}/active_frontend.txt" ]; then
        echo "Reading frontend information from active_frontend.txt" | tee -a "$LOG_FILE"
        ADAPTER=$(cat "${OUTPUT_DIR}/active_frontend.txt" | grep "adapter=" | cut -d= -f2)
        FRONTEND=$(cat "${OUTPUT_DIR}/active_frontend.txt" | grep "frontend=" | cut -d= -f2)
        
        if [ -n "$ADAPTER" ] && [ -n "$FRONTEND" ]; then
            echo "Found frontend information: adapter=$ADAPTER, frontend=$FRONTEND" | tee -a "$LOG_FILE"
        fi
    fi
fi

# Start analysis
{
    echo "==================================================="
    echo "${BAND_TYPE} Transponder Analysis Results"
    echo "==================================================="
    echo "Analysis started at: $(date)"
    echo "Using tuning frequency: $TUNING_FREQ_KHZ kHz"
    echo ""
    
    # Try adapters and frontends
    SUCCESS=0
    
    # If we have predetermined adapter/frontend (no-stop mode with frontend info)
    if [ $STOP_ENIGMA2 -eq 0 ] && [ -n "$ADAPTER" ] && [ -n "$FRONTEND" ]; then
        echo ""
        echo "Using specified adapter $ADAPTER, frontend $FRONTEND..."
        
        # Log command that will be executed
        echo "Running: $PID_EXTRACTOR $TUNING_FREQ_KHZ $POLARITY $ADAPTER $FRONTEND $SYSTEM $SYMBOL_RATE $MODULATION $FEC"
        
        # Try to force-close the device one more time directly before execution
        if [ -e "/dev/dvb/adapter${ADAPTER}/frontend${FRONTEND}" ]; then
            echo "Final device reset before execution..."
            fuser -k "/dev/dvb/adapter${ADAPTER}/frontend${FRONTEND}" 2>/dev/null || true
            # Extra sleep to let the system fully release the device
            sleep 2
        fi
        
        # Run the command directly (with much simpler execution)
        $PID_EXTRACTOR $TUNING_FREQ_KHZ $POLARITY $ADAPTER $FRONTEND $SYSTEM $SYMBOL_RATE $MODULATION $FEC
        EXIT_STATUS=$?
        
        # Check if it was killed by the timeout
        if [ $EXIT_STATUS -eq 137 ]; then
            echo "Command timed out after 30 seconds"
            EXIT_STATUS=124  # Match timeout's exit code
        fi
        echo "Command exit status: $EXIT_STATUS"
        
        # Check if the command was successful
        if [ $EXIT_STATUS -eq 0 ]; then
            SUCCESS=1
            echo ""
            echo "Success with adapter $ADAPTER, frontend $FRONTEND"
        elif [ $EXIT_STATUS -eq 124 ]; then
            echo "Command timed out after 30 seconds"
        else
            echo "Command failed with exit code $EXIT_STATUS"
        fi
    else
        # Try different adapters and frontends (default behavior)
        for adapter in 0 1; do
            for frontend in 0 1; do
                echo ""
                echo "Trying adapter $adapter, frontend $frontend..."
                
                # Log command that will be executed
                echo "Running: $PID_EXTRACTOR $TUNING_FREQ_KHZ $POLARITY $adapter $frontend $SYSTEM $SYMBOL_RATE $MODULATION $FEC"
                
                # Try to force-close the device one more time directly before execution
                if [ -e "/dev/dvb/adapter${adapter}/frontend${frontend}" ]; then
                    echo "Final device reset before execution..."
                    fuser -k "/dev/dvb/adapter${adapter}/frontend${frontend}" 2>/dev/null || true
                    # Extra sleep to let the system fully release the device
                    sleep 2
                fi
                
                # Run the command directly (with much simpler execution)
                $PID_EXTRACTOR $TUNING_FREQ_KHZ $POLARITY $adapter $frontend $SYSTEM $SYMBOL_RATE $MODULATION $FEC
                EXIT_STATUS=$?
                
                # Check if it was killed by the timeout
                if [ $EXIT_STATUS -eq 137 ]; then
                    echo "Command timed out after 30 seconds"
                    EXIT_STATUS=124  # Match timeout's exit code
                fi
                echo "Command exit status: $EXIT_STATUS"
                
                # Check if the command was successful
                if [ $EXIT_STATUS -eq 0 ]; then
                    SUCCESS=1
                    echo ""
                    echo "Success with adapter $adapter, frontend $frontend"
                    break 2
                elif [ $EXIT_STATUS -eq 124 ]; then
                    echo "Command timed out after 30 seconds"
                else
                    echo "Command failed with exit code $EXIT_STATUS"
                fi
            done
        done
    fi
    
    if [ $SUCCESS -eq 0 ]; then
        echo ""
        echo "Analysis failed on all frontends. Try again with different parameters."
    fi
    
    echo ""
    echo "Analysis completed at: $(date)"
    echo "==================================================="
} | tee -a "$RESULTS_FILE"

echo "Results saved to: $RESULTS_FILE" | tee -a "$LOG_FILE"

# Only restart Enigma2 if we stopped it
if [ $STOP_ENIGMA2 -eq 1 ]; then
    echo "Restarting Enigma2..." | tee -a "$LOG_FILE"
    init 3
    
    # Verify restart
    echo "Init 3 executed at $(date)" >> "$LOG_FILE"
    
    # Fallback restart in case init 3 fails
    sleep 10
    if ! pgrep enigma2 > /dev/null; then
        echo "Enigma2 not running after 10 seconds, trying alternative restart" >> "$LOG_FILE"
        killall -9 enigma2 2>/dev/null
        sleep 2
        /etc/init.d/enigma2 restart
    fi
else
    echo "Not restarting Enigma2 (no-stop mode)" | tee -a "$LOG_FILE"
fi

echo "Script completed at $(date)" >> "$LOG_FILE"
exit 0