from Components.config import config, ConfigSelection, ConfigSubsection, ConfigText
from Components.ConfigList import ConfigListScreen
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Components.ActionMap import ActionMap
from Components.Label import Label
from Plugins.Plugin import PluginDescriptor
from threading import Timer
import os
import hashlib

# Define available drivers
config.plugins.driverSelector = ConfigSubsection()
config.plugins.driverSelector.selectedDriver = ConfigSelection(
	default="avl6261",
	choices=[
		("t2mi-module_5.15.0", "T2MI Kernel Module"),
		("t2mi-module-tnap-mod_5.15.0", "T2MI Module with Small Symbol Rates"),
		("no-t2mi-module_5.15.0", "No-T2MI Kernel Module"),
		("avl6261-t2mi-700mhz-europe", "AVL6261-T2MI 700MHz (Europe)")
	]
)
# ✅ NEW: Store last detected driver persistently to prevent "Unknown" after reboot
config.plugins.driverSelector.lastDetectedDriver = ConfigText(default="No driver installed")

class DriverSelectorScreen(Screen, ConfigListScreen):
	skin = """
	<screen name="DriverSelectorScreen" position="center,center" size="1200,900" title="Driver Selector">
		<widget name="config" position="20,20" size="1160,400" font="Regular;40" itemHeight="80" scrollbarMode="showOnDemand" />

		<!-- New Line for Current Driver -->
		<widget name="current_driver_label" position="20,440" size="1160,40" font="Regular;38" halign="center" valign="center" />

		<!-- New Line for Checksum -->
		<widget name="current_driver_checksum" position="20,490" size="1160,40" font="Regular;34" halign="center" valign="center" />

		<widget name="key_red" position="20,740" size="560,100" halign="center" valign="center" font="Regular;40" backgroundColor="#9f1313" foregroundColor="white" />
		<widget name="key_green" position="620,740" size="560,100" halign="center" valign="center" font="Regular;40" backgroundColor="#1f771f" foregroundColor="white" />
	</screen>
	"""

	def __init__(self, session):
		Screen.__init__(self, session)
		self.session = session

		# Config list
		configList = [("Select Driver:", config.plugins.driverSelector.selectedDriver)]
		ConfigListScreen.__init__(self, configList)

		# ActionMap
		self["actions"] = ActionMap(["SetupActions", "ColorActions"], {
			"cancel": self.cancel,  # Red Button (Exit)
			"green": self.apply,    # Green Button (Apply)
			"ok": self.apply        # Pressing OK also applies
		}, -1)

		# Labels
		self["key_red"] = Label("Cancel")
		self["key_green"] = Label("Apply")

		self["current_driver_label"] = Label("")
		self["current_driver_checksum"] = Label("")

		# Instead of calling refreshUI immediately, delay it
		self.onLayoutFinish.append(self.delayedRefreshUI)

	def delayedRefreshUI(self):
		# Delay the refresh by .3 seconds to ensure the driver file is ready
		Timer(.3, self.refreshUI).start()


	def refreshUI(self):
		""" Ensures UI elements update after Enigma2 screen loads. """
		self.updateCurrentDriverLabel()
		self["key_red"].setText("Cancel")
		self["key_green"].setText("Apply")

	def updateCurrentDriverLabel(self):
		""" Show the currently installed driver and its checksum on separate lines. """
		detected_driver = self.detect_active_driver()

		# ✅ Save detected driver to Enigma2 config so it persists after reboot
		config.plugins.driverSelector.lastDetectedDriver.value = detected_driver
		config.plugins.driverSelector.lastDetectedDriver.save()

		# Split driver name and checksum
		if "(" in detected_driver:
			driver_name, checksum = detected_driver.split("(")
			checksum = "(" + checksum  # Add back the parenthesis
		else:
			driver_name = detected_driver
			checksum = ""

		self["current_driver_label"].setText("Current driver: %s" % driver_name.strip())
		self["current_driver_checksum"].setText("%s" % checksum.strip())

	def detect_active_driver(self):
		""" Detects the currently loaded driver using MD5 checksum comparison. """
		driver_checksums = {
			"AVL6261-T2MI-700MHz-Europe": "35f630e86e04ad52f76d102831b6d381",
			"No-T2MI-Kernel-Module": "f2bb20e6bada5f7d9f992f1d88486e41",
			"T2MI-Kernel-Module": "baf90056d97085f7297c420794601e39",
			"T2MI-Module-TNAP-Mod": "bf1f7d7e9deeae90b103b4cbedd9fb34"
		}

		driver_file = self.get_installed_driver_file()

		# ✅ Always assume a driver is installed if the file exists
		if not driver_file:
			return config.plugins.driverSelector.lastDetectedDriver.value

		loaded_checksum = self.get_file_checksum(driver_file)
		if loaded_checksum is None:
			return "Driver detected, but checksum unavailable"

		for name, stored_checksum in driver_checksums.items():
			if loaded_checksum == stored_checksum:
				return f"{name} (Checksum: {loaded_checksum})"

		return "Unknown avl6261 driver loaded"

	def get_installed_driver_file(self):
		""" Returns the correct path of the currently installed avl6261 driver. """
		driver_path = f"/lib/modules/{os.uname().release}/extra/avl6261.ko"
		return driver_path if os.path.exists(driver_path) else None

	def get_file_checksum(self, path):
		""" Calculate MD5 checksum of a file """
		if not os.path.exists(path):
			return None
		with open(path, "rb") as f:
			return hashlib.md5(f.read()).hexdigest()

	def apply(self):
		""" Handles the installation process when the green button is pressed. """
		self.driver = config.plugins.driverSelector.selectedDriver.value

		if self.driver == self.detect_active_driver():
			self.session.open(MessageBox, "This driver is already installed.", MessageBox.TYPE_INFO, timeout=3)
			return

		self.session.openWithCallback(self.doApply, MessageBox,
									  "Driver '%s' will be installed and the box will reboot. Continue?" % self.driver,
									  MessageBox.TYPE_YESNO)

	def doApply(self, answer):
		if answer:
			config.plugins.driverSelector.selectedDriver.save()
			self.install_driver(self.driver)
		else:
			self.close()

	def install_driver(self, driver):
		""" Installs the selected driver safely. """
		driver_source_path = f"/usr/lib/enigma2/python/Plugins/Extensions/DriverSelector/{driver}/avl6261.ko"
		driver_dest_path = f"/lib/modules/{os.uname().release}/extra/avl6261.ko"

		if os.path.exists(driver_source_path):
			os.system(f"cp {driver_source_path} {driver_dest_path}")
			os.system(f"chmod +x {driver_dest_path}")
			os.system("sync")
		else:
			print(f"⚠️ Error: Driver file {driver_source_path} not found!")

		os.system("reboot")

	def cancel(self):
		self.close()


def main(session, **kwargs):
	session.open(DriverSelectorScreen)

def Plugins(**kwargs):
	return [PluginDescriptor(
		name="Driver Selector",
		description="Select and install AVL6261/T2MI drivers",
		where=PluginDescriptor.WHERE_PLUGINMENU,
		fnc=main
	)]
