#!/bin/sh
#
# Extract PIDs script - Proof of Concept for accurate PID extraction
# 
# This script uses the pid_extractor_poc tool to directly tune to a transponder
# and extract actual service information including accurate PIDs from DVB tables
#

# Default values
ADAPTER=0
FRONTEND=0
DEMUX=0

usage() {
    echo "Usage: $0 <frequency_kHz> <polarity> [adapter] [frontend] [demux]"
    echo ""
    echo "  frequency_kHz: Transponder frequency in kHz (e.g., 4139000)"
    echo "  polarity:      0=horizontal, 1=vertical"
    echo "  adapter:       DVB adapter number (default: 0)"
    echo "  frontend:      Frontend number (default: 0)"
    echo "  demux:         Demux number (default: 0)"
    echo ""
    echo "Example: $0 4139000 0"
    echo "         (Analyzes 4139 MHz horizontal transponder)"
    echo ""
    echo "Note: This script requires root privileges to access DVB hardware."
    exit 1
}

# Check arguments
if [ $# -lt 2 ]; then
    usage
fi

FREQUENCY=$1
POLARITY=$2

# Optional parameters
if [ $# -gt 2 ]; then ADAPTER=$3; fi
if [ $# -gt 3 ]; then FRONTEND=$4; fi
if [ $# -gt 4 ]; then DEMUX=$5; fi

# Check if the pid_extractor_poc tool exists
if [ ! -x "$(command -v pid_extractor_poc)" ]; then
    # Try to find the tool in the current directory
    if [ -x "./pid_extractor_poc" ]; then
        TOOL="./pid_extractor_poc"
    else
        echo "Error: pid_extractor_poc tool not found. Make sure it's in your PATH or current directory."
        exit 1
    fi
else
    TOOL="pid_extractor_poc"
fi

# Run the PID extractor with the given parameters
echo "Analyzing transponder $FREQUENCY kHz ($([ "$POLARITY" = "0" ] && echo "Horizontal" || echo "Vertical"))"
echo "Using adapter $ADAPTER, frontend $FRONTEND, demux $DEMUX"
echo ""

# Execute the PID extractor
$TOOL $FREQUENCY $POLARITY $ADAPTER $FRONTEND $DEMUX

# Check the result
if [ $? -eq 0 ]; then
    echo ""
    echo "Analysis completed successfully."
else
    echo ""
    echo "Error: Analysis failed."
    echo "Make sure you have the correct parameters and permissions to access the DVB hardware."
    exit 1
fi